/*
 * Copyright 2002 Frdric RAYNAL <pappy@security-labs.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Niels Provos.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "sk.h"
#include "arp-eth-ip.h"
#include "version.h"
#include <stdarg.h>

/* failure notice and exit */
void
fatal(const char *format, ...)
{

    va_list ap;

    va_start(ap, format);

    if(format)
	vfprintf(stderr, format, ap);

    va_end(ap);

    fflush(stderr);

    if(sk_libnet_link)
	libnet_destroy(sk_libnet_link);
    if(sk_pkt)
	free(sk_pkt);

    exit(EXIT_FAILURE);
}

void
warning(const char *format, ...)
{

    va_list ap;

    va_start(ap, format);

    if(format)
	vfprintf(stderr, format, ap);

    va_end(ap);
    fflush(stderr);
}

arpsk_arp_pkt_t *
sk_init(int proto)
{

    arpsk_arp_pkt_t *pkt = NULL;

#if !(__WIN32__)
    struct timeval seed;
#endif

#if __WIN32__
    srand(0);
#else
    if(gettimeofday(&seed, NULL) == -1)
	fatal("** Error: cannot gettimeofday.\n");

    srandom((unsigned)((seed.tv_sec ^ seed.tv_usec) ^ getpid()));
#endif

    switch (proto)
    {
	case ARP_ETH_IP:
	    pkt = sk_malloc(sizeof(arpsk_eiarp_pkt_t));
	    sk_init_eiarp_pkt(pkt);
	    return pkt;

	default:
	    fprintf(stderr, "** Error: proto not yet supported.\n");
	    exit(EXIT_FAILURE);
    }
}

void *
sk_malloc(int size)
{

    void *mymem;

    if(size <= 0)
    {
	fatal("** Error: trying to malloc <= 0 amount of memory (%d) !!!\n",
	    size);
    }

    mymem = malloc(size);

    if(!mymem)
    {
	fatal("** Error: malloc failed, %s\n", strerror(errno));
    }

    return mymem;
}

void
show_version()
{

    printf("%s version %s (%s)\n", PACKAGE, ARPSK_VERSION, ARPSK_DATE);
    printf("Author: %s\n", ARPSK_AUTHOR);
}
